@@ -10,7 +10,7 @@ class TestVersionCompatibility(unittest.TestCase):
             warnings.simplefilter("always")
 
             # strict=True is deprecated in Py33+
-            conn = HTTPConnection('localhost', 12345, strict=True)
+            HTTPConnection('localhost', 12345, strict=True)
 
             if w:
                 self.fail('HTTPConnection raised warning on strict=True: %r' % w[0].message)
@@ -18,6 +18,6 @@ class TestVersionCompatibility(unittest.TestCase):
     def test_connection_source_address(self):
         try:
             # source_address does not exist in Py26-
-            conn = HTTPConnection('localhost', 12345, source_address='127.0.0.1')
+            HTTPConnection('localhost', 12345, source_address='127.0.0.1')
         except TypeError as e:
             self.fail('HTTPConnection raised TypeError on source_adddress: %r' % e)
