@@ -16,10 +16,12 @@ class RetryTest(unittest.TestCase):
     def test_string(self):
         """ Retry string representation looks the way we expect """
         retry = Retry()
-        self.assertEqual(str(retry), 'Retry(total=10, connect=None, read=None, redirect=None)')
+        self.assertEqual(str(retry),
+                         'Retry(total=10, connect=None, read=None, redirect=None, status=None)')
         for _ in range(3):
-            retry = retry.increment()
-        self.assertEqual(str(retry), 'Retry(total=7, connect=None, read=None, redirect=None)')
+            retry = retry.increment(method='GET')
+        self.assertEqual(str(retry),
+                         'Retry(total=7, connect=None, read=None, redirect=None, status=None)')
 
     def test_retry_both_specified(self):
         """Total can win if it's lower than the connect value"""
@@ -45,9 +47,9 @@ class RetryTest(unittest.TestCase):
         """ A lower read timeout than the total is honored """
         error = ReadTimeoutError(None, "/", "read timed out")
         retry = Retry(read=2, total=3)
-        retry = retry.increment(error=error)
-        retry = retry.increment(error=error)
-        self.assertRaises(MaxRetryError, retry.increment, error=error)
+        retry = retry.increment(method='GET', error=error)
+        retry = retry.increment(method='GET', error=error)
+        self.assertRaises(MaxRetryError, retry.increment, method='GET', error=error)
 
     def test_retry_total_none(self):
         """ if Total is none, connect error should take precedence """
@@ -63,9 +65,9 @@ class RetryTest(unittest.TestCase):
 
         error = ReadTimeoutError(None, "/", "read timed out")
         retry = Retry(connect=2, total=None)
-        retry = retry.increment(error=error)
-        retry = retry.increment(error=error)
-        retry = retry.increment(error=error)
+        retry = retry.increment(method='GET', error=error)
+        retry = retry.increment(method='GET', error=error)
+        retry = retry.increment(method='GET', error=error)
         self.assertFalse(retry.is_exhausted())
 
     def test_retry_default(self):
@@ -93,65 +95,77 @@ class RetryTest(unittest.TestCase):
         error = ReadTimeoutError(None, "/", "read timed out")
         retry = Retry(read=0)
         try:
-            retry.increment(error=error)
+            retry.increment(method='GET', error=error)
             self.fail("Failed to raise error.")
         except MaxRetryError as e:
             self.assertEqual(e.reason, error)
 
+    def test_status_counter(self):
+        resp = HTTPResponse(status=400)
+        retry = Retry(status=2)
+        retry = retry.increment(response=resp)
+        retry = retry.increment(response=resp)
+        try:
+            retry.increment(response=resp)
+            self.fail("Failed to raise error.")
+        except MaxRetryError as e:
+            self.assertEqual(str(e.reason),
+                             ResponseError.SPECIFIC_ERROR.format(status_code=400))
+
     def test_backoff(self):
         """ Backoff is computed correctly """
         max_backoff = Retry.BACKOFF_MAX
 
         retry = Retry(total=100, backoff_factor=0.2)
-        self.assertEqual(retry.get_backoff_time(), 0) # First request
+        self.assertEqual(retry.get_backoff_time(), 0)  # First request
 
-        retry = retry.increment()
-        self.assertEqual(retry.get_backoff_time(), 0) # First retry
+        retry = retry.increment(method='GET')
+        self.assertEqual(retry.get_backoff_time(), 0)  # First retry
 
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.backoff_factor, 0.2)
         self.assertEqual(retry.total, 98)
-        self.assertEqual(retry.get_backoff_time(), 0.4) # Start backoff
+        self.assertEqual(retry.get_backoff_time(), 0.4)  # Start backoff
 
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.get_backoff_time(), 0.8)
 
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.get_backoff_time(), 1.6)
 
         for i in xrange(10):
-            retry = retry.increment()
+            retry = retry.increment(method='GET')
 
         self.assertEqual(retry.get_backoff_time(), max_backoff)
 
     def test_zero_backoff(self):
         retry = Retry()
         self.assertEqual(retry.get_backoff_time(), 0)
-        retry = retry.increment()
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.get_backoff_time(), 0)
 
     def test_backoff_reset_after_redirect(self):
         retry = Retry(total=100, redirect=5, backoff_factor=0.2)
-        retry = retry.increment()
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.get_backoff_time(), 0.4)
         redirect_response = HTTPResponse(status=302, headers={'location': 'test'})
-        retry = retry.increment(response=redirect_response)
+        retry = retry.increment(method='GET', response=redirect_response)
         self.assertEqual(retry.get_backoff_time(), 0)
-        retry = retry.increment()
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
+        retry = retry.increment(method='GET')
         self.assertEqual(retry.get_backoff_time(), 0.4)
 
     def test_sleep(self):
         # sleep a very small amount of time so our code coverage is happy
         retry = Retry(backoff_factor=0.0001)
-        retry = retry.increment()
-        retry = retry.increment()
+        retry = retry.increment(method='GET')
+        retry = retry.increment(method='GET')
         retry.sleep()
 
     def test_status_forcelist(self):
-        retry = Retry(status_forcelist=xrange(500,600))
+        retry = Retry(status_forcelist=xrange(500, 600))
         self.assertFalse(retry.is_retry('GET', status_code=200))
         self.assertFalse(retry.is_retry('GET', status_code=400))
         self.assertTrue(retry.is_retry('GET', status_code=500))
@@ -178,16 +192,17 @@ class RetryTest(unittest.TestCase):
     def test_exhausted(self):
         self.assertFalse(Retry(0).is_exhausted())
         self.assertTrue(Retry(-1).is_exhausted())
-        self.assertEqual(Retry(1).increment().total, 0)
+        self.assertEqual(Retry(1).increment(method='GET').total, 0)
 
     def test_disabled(self):
-        self.assertRaises(MaxRetryError, Retry(-1).increment)
-        self.assertRaises(MaxRetryError, Retry(0).increment)
+        self.assertRaises(MaxRetryError, Retry(-1).increment, method='GET')
+        self.assertRaises(MaxRetryError, Retry(0).increment, method='GET')
 
     def test_error_message(self):
         retry = Retry(total=0)
         try:
-            retry = retry.increment(error=ReadTimeoutError(None, "/", "read timed out"))
+            retry = retry.increment(method='GET',
+                                    error=ReadTimeoutError(None, "/", "read timed out"))
             raise AssertionError("Should have raised a MaxRetryError")
         except MaxRetryError as e:
             assert 'Caused by redirect' not in str(e)
@@ -225,17 +240,27 @@ class RetryTest(unittest.TestCase):
             self.assertEqual(str(e.reason), 'conntimeout')
 
     def test_history(self):
-        retry = Retry(total=10)
+        retry = Retry(total=10, method_whitelist=frozenset(['GET', 'POST']))
         self.assertEqual(retry.history, tuple())
         connection_error = ConnectTimeoutError('conntimeout')
         retry = retry.increment('GET', '/test1', None, connection_error)
-        self.assertEqual(retry.history, (RequestHistory('GET', '/test1', connection_error, None, None),))
+        history = (RequestHistory('GET', '/test1', connection_error, None, None),)
+        self.assertEqual(retry.history, history)
+
         read_error = ReadTimeoutError(None, "/test2", "read timed out")
         retry = retry.increment('POST', '/test2', None, read_error)
-        self.assertEqual(retry.history, (RequestHistory('GET', '/test1', connection_error, None, None),
-                                         RequestHistory('POST', '/test2', read_error, None, None)))
+        history = (RequestHistory('GET', '/test1', connection_error, None, None),
+                   RequestHistory('POST', '/test2', read_error, None, None))
+        self.assertEqual(retry.history, history)
+
         response = HTTPResponse(status=500)
         retry = retry.increment('GET', '/test3', response, None)
-        self.assertEqual(retry.history, (RequestHistory('GET', '/test1', connection_error, None, None),
-                                         RequestHistory('POST', '/test2', read_error, None, None),
-                                         RequestHistory('GET', '/test3', None, 500, None)))
+        history = (RequestHistory('GET', '/test1', connection_error, None, None),
+                   RequestHistory('POST', '/test2', read_error, None, None),
+                   RequestHistory('GET', '/test3', None, 500, None))
+        self.assertEqual(retry.history, history)
+
+    def test_retry_method_not_in_whitelist(self):
+        error = ReadTimeoutError(None, "/", "read timed out")
+        retry = Retry()
+        self.assertRaises(ReadTimeoutError, retry.increment, method='POST', error=error)
