@@ -1,6 +1,7 @@
 import hashlib
 import warnings
 import logging
+import io
 import unittest
 import ssl
 import socket
@@ -9,7 +10,7 @@ from itertools import chain
 from mock import patch, Mock
 
 from urllib3 import add_stderr_logger, disable_warnings
-from urllib3.util.request import make_headers
+from urllib3.util.request import make_headers, rewind_body, _FAILEDTELL
 from urllib3.util.retry import Retry
 from urllib3.util.timeout import Timeout
 from urllib3.util.url import (
@@ -28,9 +29,9 @@ from urllib3.exceptions import (
     LocationParseError,
     TimeoutStateError,
     InsecureRequestWarning,
-    SSLError,
     SNIMissingWarning,
     InvalidHeader,
+    UnrewindableBodyError,
 )
 from urllib3.util.connection import (
     allowed_gai_family,
@@ -46,6 +47,7 @@ from . import clear_warnings
 # numbers used for timeouts
 TIMEOUT_EPOCH = 1000
 
+
 class TestUtil(unittest.TestCase):
     def test_get_host(self):
         url_host_map = {
@@ -81,8 +83,10 @@ class TestUtil(unittest.TestCase):
             'http://[2a00:1450:4001:c01::67]:80/test': ('http', '[2a00:1450:4001:c01::67]', 80),
 
             # More IPv6 from http://www.ietf.org/rfc/rfc2732.txt
-            'http://[fedc:ba98:7654:3210:fedc:ba98:7654:3210]:8000/index.html': ('http', '[fedc:ba98:7654:3210:fedc:ba98:7654:3210]', 8000),
-            'http://[1080:0:0:0:8:800:200c:417a]/index.html': ('http', '[1080:0:0:0:8:800:200c:417a]', None),
+            'http://[fedc:ba98:7654:3210:fedc:ba98:7654:3210]:8000/index.html': (
+                'http', '[fedc:ba98:7654:3210:fedc:ba98:7654:3210]', 8000),
+            'http://[1080:0:0:0:8:800:200c:417a]/index.html': (
+                'http', '[1080:0:0:0:8:800:200c:417a]', None),
             'http://[3ffe:2a00:100:7031::1]': ('http', '[3ffe:2a00:100:7031::1]', None),
             'http://[1080::8:800:200c:417a]/foo': ('http', '[1080::8:800:200c:417a]', None),
             'http://[::192.9.5.5]/ipng': ('http', '[::192.9.5.5]', None),
@@ -107,7 +111,10 @@ class TestUtil(unittest.TestCase):
             self.assertRaises(LocationParseError, get_host, location)
 
     def test_host_normalization(self):
-        """Asserts the scheme and host is normalized to lower-case."""
+        """
+        Asserts the scheme and hosts with a normalizable scheme are
+        converted to lower-case.
+        """
         url_host_map = {
             # Hosts
             'HTTP://GOOGLE.COM/mail/': ('http', 'google.com', None),
@@ -117,8 +124,13 @@ class TestUtil(unittest.TestCase):
             '173.194.35.7': ('http', '173.194.35.7', None),
             'HTTP://173.194.35.7': ('http', '173.194.35.7', None),
             'HTTP://[2a00:1450:4001:c01::67]:80/test': ('http', '[2a00:1450:4001:c01::67]', 80),
-            'HTTP://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:8000/index.html': ('http', '[fedc:ba98:7654:3210:fedc:ba98:7654:3210]', 8000),
-            'HTTPS://[1080:0:0:0:8:800:200c:417A]/index.html': ('https', '[1080:0:0:0:8:800:200c:417a]', None),
+            'HTTP://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:8000/index.html': (
+                'http', '[fedc:ba98:7654:3210:fedc:ba98:7654:3210]', 8000),
+            'HTTPS://[1080:0:0:0:8:800:200c:417A]/index.html': (
+                'https', '[1080:0:0:0:8:800:200c:417a]', None),
+            'abOut://eXamPlE.com?info=1': ('about', 'eXamPlE.com', None),
+            'http+UNIX://%2fvar%2frun%2fSOCKET/path': (
+                'http+unix', '%2fvar%2frun%2fSOCKET', None),
         }
         for url, expected_host in url_host_map.items():
             returned_host = get_host(url)
@@ -128,7 +140,8 @@ class TestUtil(unittest.TestCase):
         """Assert parse_url normalizes the scheme/host, and only the scheme/host"""
         test_urls = [
             ('HTTP://GOOGLE.COM/MAIL/', 'http://google.com/MAIL/'),
-            ('HTTP://JeremyCline:Hunter2@Example.com:8080/', 'http://JeremyCline:Hunter2@example.com:8080/'),
+            ('HTTP://JeremyCline:Hunter2@Example.com:8080/',
+             'http://JeremyCline:Hunter2@example.com:8080/'),
             ('HTTPS://Example.Com/?Key=Value', 'https://example.com/?Key=Value'),
             ('Https://Example.Com/#Fragment', 'https://example.com/#Fragment'),
         ]
@@ -136,34 +149,39 @@ class TestUtil(unittest.TestCase):
             actual_normalized_url = parse_url(url).url
             self.assertEqual(actual_normalized_url, expected_normalized_url)
 
-    parse_url_host_map = {
-        'http://google.com/mail': Url('http', host='google.com', path='/mail'),
-        'http://google.com/mail/': Url('http', host='google.com', path='/mail/'),
-        'http://google.com/mail': Url('http', host='google.com', path='mail'),
-        'google.com/mail': Url(host='google.com', path='/mail'),
-        'http://google.com/': Url('http', host='google.com', path='/'),
-        'http://google.com': Url('http', host='google.com'),
-        'http://google.com?foo': Url('http', host='google.com', path='', query='foo'),
+    parse_url_host_map = [
+        ('http://google.com/mail', Url('http', host='google.com', path='/mail')),
+        ('http://google.com/mail/', Url('http', host='google.com', path='/mail/')),
+        ('http://google.com/mail', Url('http', host='google.com', path='mail')),
+        ('google.com/mail', Url(host='google.com', path='/mail')),
+        ('http://google.com/', Url('http', host='google.com', path='/')),
+        ('http://google.com', Url('http', host='google.com')),
+        ('http://google.com?foo', Url('http', host='google.com', path='', query='foo')),
 
         # Path/query/fragment
-        '': Url(),
-        '/': Url(path='/'),
-        '#?/!google.com/?foo#bar': Url(path='', fragment='?/!google.com/?foo#bar'),
-        '/foo': Url(path='/foo'),
-        '/foo?bar=baz': Url(path='/foo', query='bar=baz'),
-        '/foo?bar=baz#banana?apple/orange': Url(path='/foo', query='bar=baz', fragment='banana?apple/orange'),
+        ('', Url()),
+        ('/', Url(path='/')),
+        ('#?/!google.com/?foo#bar', Url(path='', fragment='?/!google.com/?foo#bar')),
+        ('/foo', Url(path='/foo')),
+        ('/foo?bar=baz', Url(path='/foo', query='bar=baz')),
+        ('/foo?bar=baz#banana?apple/orange', Url(path='/foo',
+                                                 query='bar=baz',
+                                                 fragment='banana?apple/orange')),
 
         # Port
-        'http://google.com/': Url('http', host='google.com', path='/'),
-        'http://google.com:80/': Url('http', host='google.com', port=80, path='/'),
-        'http://google.com:80': Url('http', host='google.com', port=80),
+        ('http://google.com/', Url('http', host='google.com', path='/')),
+        ('http://google.com:80/', Url('http', host='google.com', port=80, path='/')),
+        ('http://google.com:80', Url('http', host='google.com', port=80)),
 
         # Auth
-        'http://foo:bar@localhost/': Url('http', auth='foo:bar', host='localhost', path='/'),
-        'http://foo@localhost/': Url('http', auth='foo', host='localhost', path='/'),
-        'http://foo:bar@baz@localhost/': Url('http', auth='foo:bar@baz', host='localhost', path='/'),
-        'http://@': Url('http', host=None, auth='')
-    }
+        ('http://foo:bar@localhost/', Url('http', auth='foo:bar', host='localhost', path='/')),
+        ('http://foo@localhost/', Url('http', auth='foo', host='localhost', path='/')),
+        ('http://foo:bar@baz@localhost/', Url('http',
+                                              auth='foo:bar@baz',
+                                              host='localhost',
+                                              path='/')),
+        ('http://@', Url('http', host=None, auth=''))
+    ]
 
     non_round_tripping_parse_url_host_map = {
         # Path/query/fragment
@@ -177,12 +195,13 @@ class TestUtil(unittest.TestCase):
         }
 
     def test_parse_url(self):
-        for url, expected_Url in chain(self.parse_url_host_map.items(), self.non_round_tripping_parse_url_host_map.items()):
+        for url, expected_Url in chain(self.parse_url_host_map,
+                                       self.non_round_tripping_parse_url_host_map.items()):
             returned_Url = parse_url(url)
             self.assertEqual(returned_Url, expected_Url)
 
     def test_unparse_url(self):
-        for url, expected_Url in self.parse_url_host_map.items():
+        for url, expected_Url in self.parse_url_host_map:
             self.assertEqual(url, expected_Url.url)
 
     def test_parse_url_invalid_IPv6(self):
@@ -256,6 +275,41 @@ class TestUtil(unittest.TestCase):
             make_headers(disable_cache=True),
             {'cache-control': 'no-cache'})
 
+    def test_rewind_body(self):
+        body = io.BytesIO(b'test data')
+        self.assertEqual(body.read(), b'test data')
+
+        # Assert the file object has been consumed
+        self.assertEqual(body.read(), b'')
+
+        # Rewind it back to just be b'data'
+        rewind_body(body, 5)
+        self.assertEqual(body.read(), b'data')
+
+    def test_rewind_body_failed_tell(self):
+        body = io.BytesIO(b'test data')
+        body.read()  # Consume body
+
+        # Simulate failed tell()
+        body_pos = _FAILEDTELL
+        self.assertRaises(UnrewindableBodyError, rewind_body, body, body_pos)
+
+    def test_rewind_body_bad_position(self):
+        body = io.BytesIO(b'test data')
+        body.read()  # Consume body
+
+        # Pass non-integer position
+        self.assertRaises(ValueError, rewind_body, body, None)
+        self.assertRaises(ValueError, rewind_body, body, object())
+
+    def test_rewind_body_failed_seek(self):
+        class BadSeek():
+
+            def seek(self, pos, offset=0):
+                raise IOError
+
+        self.assertRaises(UnrewindableBodyError, rewind_body, BadSeek(), 2)
+
     def test_split_first(self):
         test_cases = {
             ('abcd', 'b'): ('a', 'cd', 'b'),
@@ -269,7 +323,7 @@ class TestUtil(unittest.TestCase):
             self.assertEqual(output, expected)
 
     def test_add_stderr_logger(self):
-        handler = add_stderr_logger(level=logging.INFO) # Don't actually print debug
+        handler = add_stderr_logger(level=logging.INFO)  # Don't actually print debug
         logger = logging.getLogger('urllib3')
         self.assertTrue(handler in logger.handlers)
 
@@ -334,7 +388,6 @@ class TestUtil(unittest.TestCase):
         except ValueError as e:
             self.assertTrue('int, float or None' in str(e))
 
-
     @patch('urllib3.util.timeout.current_time')
     def test_timeout(self, current_time):
         timeout = Timeout(total=3)
@@ -374,14 +427,12 @@ class TestUtil(unittest.TestCase):
         timeout = Timeout(5)
         self.assertEqual(timeout.total, 5)
 
-
     def test_timeout_str(self):
         timeout = Timeout(connect=1, read=2, total=3)
         self.assertEqual(str(timeout), "Timeout(connect=1, read=2, total=3)")
         timeout = Timeout(connect=1, read=None, total=3)
         self.assertEqual(str(timeout), "Timeout(connect=1, read=None, total=3)")
 
-
     @patch('urllib3.util.timeout.current_time')
     def test_timeout_elapsed(self, current_time):
         current_time.return_value = TIMEOUT_EPOCH
