@@ -13,14 +13,16 @@ from dummyserver.testcase import (
 )
 from dummyserver.server import (DEFAULT_CA, DEFAULT_CA_BAD, DEFAULT_CERTS,
                                 NO_SAN_CERTS, NO_SAN_CA, DEFAULT_CA_DIR,
-                                IPV6_ADDR_CERTS, IPV6_ADDR_CA, HAS_IPV6)
+                                IPV6_ADDR_CERTS, IPV6_ADDR_CA, HAS_IPV6,
+                                IP_SAN_CERTS)
 
 from test import (
     onlyPy26OrOlder,
     onlyPy279OrNewer,
+    notSecureTransport,
+    onlyPy27OrNewerOrNonWindows,
     requires_network,
     TARPIT_HOST,
-    clear_warnings,
 )
 from urllib3 import HTTPSConnectionPool
 from urllib3.connection import (
@@ -30,7 +32,6 @@ from urllib3.connection import (
 )
 from urllib3.exceptions import (
     SSLError,
-    ReadTimeoutError,
     ConnectTimeoutError,
     InsecureRequestWarning,
     SystemTimeWarning,
@@ -51,10 +52,10 @@ log.setLevel(logging.NOTSET)
 log.addHandler(logging.StreamHandler(sys.stdout))
 
 
-
 class TestHTTPS(HTTPSDummyServerTestCase):
     def setUp(self):
         self._pool = HTTPSConnectionPool(self.host, self.port)
+        self.addCleanup(self._pool.close)
 
     def test_simple(self):
         r = self._pool.request('GET', '/')
@@ -77,6 +78,62 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             r = https_pool.request('GET', '/')
             self.assertEqual(r.status, 200)
 
+            # Modern versions of Python, or systems using PyOpenSSL, don't
+            # emit warnings.
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+                self.assertFalse(warn.called, warn.call_args_list)
+            else:
+                self.assertTrue(warn.called)
+                if util.HAS_SNI:
+                    call = warn.call_args_list[0]
+                else:
+                    call = warn.call_args_list[1]
+                error = call[0][1]
+                self.assertEqual(error, InsecurePlatformWarning)
+
+    def test_verified_with_context(self):
+        ctx = util.ssl_.create_urllib3_context(cert_reqs=ssl.CERT_REQUIRED)
+        ctx.load_verify_locations(cafile=DEFAULT_CA)
+        https_pool = HTTPSConnectionPool(self.host, self.port,
+                                         ssl_context=ctx)
+        self.addCleanup(https_pool.close)
+
+        conn = https_pool._new_conn()
+        self.assertEqual(conn.__class__, VerifiedHTTPSConnection)
+
+        with mock.patch('warnings.warn') as warn:
+            r = https_pool.request('GET', '/')
+            self.assertEqual(r.status, 200)
+
+            # Modern versions of Python, or systems using PyOpenSSL, don't
+            # emit warnings.
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+                self.assertFalse(warn.called, warn.call_args_list)
+            else:
+                self.assertTrue(warn.called)
+                if util.HAS_SNI:
+                    call = warn.call_args_list[0]
+                else:
+                    call = warn.call_args_list[1]
+                error = call[0][1]
+                self.assertEqual(error, InsecurePlatformWarning)
+
+    def test_context_combines_with_ca_certs(self):
+        ctx = util.ssl_.create_urllib3_context(cert_reqs=ssl.CERT_REQUIRED)
+        https_pool = HTTPSConnectionPool(self.host, self.port,
+                                         ca_certs=DEFAULT_CA,
+                                         ssl_context=ctx)
+        self.addCleanup(https_pool.close)
+
+        conn = https_pool._new_conn()
+        self.assertEqual(conn.__class__, VerifiedHTTPSConnection)
+
+        with mock.patch('warnings.warn') as warn:
+            r = https_pool.request('GET', '/')
+            self.assertEqual(r.status, 200)
+
+            # Modern versions of Python, or systems using PyOpenSSL, don't
+            # emit warnings.
             if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
                 self.assertFalse(warn.called, warn.call_args_list)
             else:
@@ -89,10 +146,12 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 self.assertEqual(error, InsecurePlatformWarning)
 
     @onlyPy279OrNewer
+    @notSecureTransport
     def test_ca_dir_verified(self):
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_cert_dir=DEFAULT_CA_DIR)
+        self.addCleanup(https_pool.close)
 
         conn = https_pool._new_conn()
         self.assertEqual(conn.__class__, VerifiedHTTPSConnection)
@@ -113,16 +172,22 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
+
         try:
             https_pool.request('GET', '/')
             self.fail("Didn't raise SSL invalid common name")
         except SSLError as e:
-            self.assertTrue("doesn't match" in str(e))
+            self.assertTrue(
+                "doesn't match" in str(e) or
+                "certificate verify failed" in str(e)
+            )
 
     def test_verified_with_bad_ca_certs(self):
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA_BAD)
+        self.addCleanup(https_pool.close)
 
         try:
             https_pool.request('GET', '/')
@@ -136,6 +201,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         # default is cert_reqs=None which is ssl.CERT_NONE
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          cert_reqs='CERT_REQUIRED')
+        self.addCleanup(https_pool.close)
 
         try:
             https_pool.request('GET', '/')
@@ -145,14 +211,17 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             # there is a different error message depending on whether or
             # not pyopenssl is injected
             self.assertTrue('No root certificates specified' in str(e) or
-                            'certificate verify failed' in str(e),
-                            "Expected 'No root certificates specified' or "
-                            "'certificate verify failed', "
+                            'certificate verify failed' in str(e) or
+                            'invalid certificate chain' in str(e),
+                            "Expected 'No root certificates specified',  "
+                            "'certificate verify failed', or "
+                            "'invalid certificate chain', "
                             "instead got: %r" % e)
 
     def test_no_ssl(self):
         pool = HTTPSConnectionPool(self.host, self.port)
         pool.ConnectionCls = None
+        self.addCleanup(pool.close)
         self.assertRaises(SSLError, pool._new_conn)
         self.assertRaises(SSLError, pool.request, 'GET', '/')
 
@@ -160,26 +229,16 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         """ Test that bare HTTPSConnection can connect, make requests """
         pool = HTTPSConnectionPool(self.host, self.port)
         pool.ConnectionCls = UnverifiedHTTPSConnection
+        self.addCleanup(pool.close)
 
         with mock.patch('warnings.warn') as warn:
             r = pool.request('GET', '/')
             self.assertEqual(r.status, 200)
             self.assertTrue(warn.called)
 
-            call, = warn.call_args_list
-            category = call[0][1]
-            self.assertEqual(category, InsecureRequestWarning)
-
-    def test_ssl_unverified_with_ca_certs(self):
-        pool = HTTPSConnectionPool(self.host, self.port,
-                                   cert_reqs='CERT_NONE',
-                                   ca_certs=DEFAULT_CA_BAD)
-
-        with mock.patch('warnings.warn') as warn:
-            r = pool.request('GET', '/')
-            self.assertEqual(r.status, 200)
-            self.assertTrue(warn.called)
-
+            # Modern versions of Python, or systems using PyOpenSSL, only emit
+            # the unverified warning. Older systems may also emit other
+            # warnings, which we want to ignore here.
             calls = warn.call_args_list
             if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
                 category = calls[0][0][1]
@@ -189,37 +248,34 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 category = calls[2][0][1]
             self.assertEqual(category, InsecureRequestWarning)
 
-    @requires_network
-    def test_ssl_verified_with_platform_ca_certs(self):
-        """
-        We should rely on the platform CA file to validate authenticity of SSL
-        certificates. Since this file is used by many components of the OS,
-        such as curl, apt-get, etc., we decided to not touch it, in order to
-        not compromise the security of the OS running the test suite (typically
-        urllib3 developer's OS).
+    def test_ssl_unverified_with_ca_certs(self):
+        pool = HTTPSConnectionPool(self.host, self.port,
+                                   cert_reqs='CERT_NONE',
+                                   ca_certs=DEFAULT_CA_BAD)
+        self.addCleanup(pool.close)
 
-        This test assumes that httpbin.org uses a certificate signed by a well
-        known Certificate Authority.
-        """
-        try:
-            import urllib3.contrib.pyopenssl
-        except ImportError:
-            raise SkipTest('Test requires PyOpenSSL')
-        if (urllib3.connection.ssl_wrap_socket is
-                urllib3.contrib.pyopenssl.orig_connection_ssl_wrap_socket):
-            # Not patched
-            raise SkipTest('Test should only be run after PyOpenSSL '
-                           'monkey patching')
+        with mock.patch('warnings.warn') as warn:
+            r = pool.request('GET', '/')
+            self.assertEqual(r.status, 200)
+            self.assertTrue(warn.called)
 
-        https_pool = HTTPSConnectionPool('httpbin.org', 443,
-                                         cert_reqs=ssl.CERT_REQUIRED)
-
-        https_pool.request('HEAD', '/')
+            # Modern versions of Python, or systems using PyOpenSSL, only emit
+            # the unverified warning. Older systems may also emit other
+            # warnings, which we want to ignore here.
+            calls = warn.call_args_list
+            if sys.version_info >= (2, 7, 9) or util.IS_PYOPENSSL:
+                category = calls[0][0][1]
+            elif util.HAS_SNI:
+                category = calls[1][0][1]
+            else:
+                category = calls[2][0][1]
+            self.assertEqual(category, InsecureRequestWarning)
 
     def test_assert_hostname_false(self):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_hostname = False
         https_pool.request('GET', '/')
@@ -228,6 +284,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_hostname = 'localhost'
         https_pool.request('GET', '/')
@@ -236,6 +293,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = 'F2:06:5A:42:10:3F:45:1C:17:FE:E6:' \
                                         '07:1E:8A:86:E5'
@@ -246,6 +304,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = '92:81:FE:85:F7:0C:26:60:EC:D6:B3:' \
                                         'BF:93:CF:F9:71:CC:07:7D:0A'
@@ -255,6 +314,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('localhost', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = ('C5:4D:0B:83:84:89:2E:AE:B4:58:BB:12:'
                                          'F7:A6:C4:76:05:03:88:D8:57:65:51:F3:'
@@ -265,6 +325,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = 'AA:AA:AA:AA:AA:AAAA:AA:AAAA:AA:' \
                                         'AA:AA:AA:AA:AA:AA:AA:AA:AA'
@@ -285,6 +346,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_NONE',
                                          ca_certs=DEFAULT_CA_BAD)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = 'AA:AA:AA:AA:AA:AAAA:AA:AAAA:AA:' \
                                         'AA:AA:AA:AA:AA:AA:AA:AA:AA'
@@ -294,15 +356,22 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_NONE',
                                          ca_certs=DEFAULT_CA_BAD)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = '92:81:FE:85:F7:0C:26:60:EC:D6:B3:' \
                                         'BF:93:CF:F9:71:CC:07:7D:0A'
         https_pool.request('GET', '/')
 
+    @notSecureTransport
     def test_good_fingerprint_and_hostname_mismatch(self):
+        # This test doesn't run with SecureTransport because we don't turn off
+        # hostname validation without turning off all validation, which this
+        # test doesn't do (deliberately). We should revisit this if we make
+        # new decisions.
         https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
 
         https_pool.assert_fingerprint = '92:81:FE:85:F7:0C:26:60:EC:D6:B3:' \
                                         'BF:93:CF:F9:71:CC:07:7D:0A'
@@ -314,17 +383,20 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool(TARPIT_HOST, self.port,
                                          timeout=timeout, retries=False,
                                          cert_reqs='CERT_REQUIRED')
+        self.addCleanup(https_pool.close)
 
         timeout = Timeout(total=None, connect=0.001)
         https_pool = HTTPSConnectionPool(TARPIT_HOST, self.port,
                                          timeout=timeout, retries=False,
                                          cert_reqs='CERT_REQUIRED')
+        self.addCleanup(https_pool.close)
         self.assertRaises(ConnectTimeoutError, https_pool.request, 'GET', '/')
 
         timeout = Timeout(read=0.001)
         https_pool = HTTPSConnectionPool(self.host, self.port,
                                          timeout=timeout, retries=False,
                                          cert_reqs='CERT_REQUIRED')
+        self.addCleanup(https_pool.close)
         https_pool.ca_certs = DEFAULT_CA
         https_pool.assert_fingerprint = '92:81:FE:85:F7:0C:26:60:EC:D6:B3:' \
                                         'BF:93:CF:F9:71:CC:07:7D:0A'
@@ -332,6 +404,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         timeout = Timeout(total=None)
         https_pool = HTTPSConnectionPool(self.host, self.port, timeout=timeout,
                                          cert_reqs='CERT_NONE')
+        self.addCleanup(https_pool.close)
         https_pool.request('GET', '/')
 
     def test_tunnel(self):
@@ -339,10 +412,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         timeout = Timeout(total=None)
         https_pool = HTTPSConnectionPool(self.host, self.port, timeout=timeout,
                                          cert_reqs='CERT_NONE')
+        self.addCleanup(https_pool.close)
         conn = https_pool._new_conn()
         try:
             conn.set_tunnel(self.host, self.port)
-        except AttributeError: # python 2.6
+        except AttributeError:  # python 2.6
             conn._set_tunnel(self.host, self.port)
         conn._tunnel = mock.Mock()
         https_pool._make_request(conn, 'GET', '/')
@@ -367,6 +441,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                                              timeout=timeout,
                                              retries=False,
                                              cert_reqs=cert_reqs)
+            self.addCleanup(https_pool.close)
             return https_pool
 
         https_pool = new_pool(Timeout(connect=0.001))
@@ -390,8 +465,10 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
         conn = VerifiedHTTPSConnection(self.host, self.port)
         https_pool = HTTPSConnectionPool(self.host, self.port,
-                cert_reqs='CERT_REQUIRED', ca_certs=DEFAULT_CA,
-                assert_fingerprint=fingerprint)
+                                         cert_reqs='CERT_REQUIRED',
+                                         ca_certs=DEFAULT_CA,
+                                         assert_fingerprint=fingerprint)
+        self.addCleanup(https_pool.close)
 
         https_pool._make_request(conn, 'GET', '/')
 
@@ -430,20 +507,13 @@ class TestHTTPS_TLSv1(HTTPSDummyServerTestCase):
 
     def setUp(self):
         self._pool = HTTPSConnectionPool(self.host, self.port)
+        self.addCleanup(self._pool.close)
 
-    def test_set_ssl_version_to_sslv3(self):
-        self._pool.ssl_version = ssl.PROTOCOL_SSLv3
-        self.assertRaises(SSLError, self._pool.request, 'GET', '/')
-
-    def test_ssl_version_as_string(self):
-        self._pool.ssl_version = 'PROTOCOL_SSLv3'
-        self.assertRaises(SSLError, self._pool.request, 'GET', '/')
-
-    def test_ssl_version_as_short_string(self):
-        self._pool.ssl_version = 'SSLv3'
-        self.assertRaises(SSLError, self._pool.request, 'GET', '/')
-
+    @onlyPy27OrNewerOrNonWindows
     def test_discards_connection_on_sslerror(self):
+        # This test is skipped on Windows for Python 2.6 because we suspect there
+        # is an issue with the OpenSSL for Python 2.6 on Windows.
+
         self._pool.cert_reqs = 'CERT_REQUIRED'
         self.assertRaises(SSLError, self._pool.request, 'GET', '/')
         self._pool.ca_certs = DEFAULT_CA
@@ -451,7 +521,7 @@ class TestHTTPS_TLSv1(HTTPSDummyServerTestCase):
 
     def test_set_cert_default_cert_required(self):
         conn = VerifiedHTTPSConnection(self.host, self.port)
-        conn.set_cert(ca_certs='/etc/ssl/certs/custom.pem')
+        conn.set_cert(ca_certs=DEFAULT_CA)
         self.assertEqual(conn.cert_reqs, 'CERT_REQUIRED')
 
 
@@ -465,11 +535,29 @@ class TestHTTPS_NoSAN(HTTPSDummyServerTestCase):
             https_pool = HTTPSConnectionPool(self.host, self.port,
                                              cert_reqs='CERT_REQUIRED',
                                              ca_certs=NO_SAN_CA)
+            self.addCleanup(https_pool.close)
             r = https_pool.request('GET', '/')
             self.assertEqual(r.status, 200)
             self.assertTrue(warn.called)
 
 
+class TestHTTPS_IPSAN(HTTPSDummyServerTestCase):
+    certs = IP_SAN_CERTS
+
+    def test_can_validate_ip_san(self):
+        """Ensure that urllib3 can validate SANs with IP addresses in them."""
+        try:
+            import ipaddress  # noqa: F401
+        except ImportError:
+            raise SkipTest("Only runs on systems with an ipaddress module")
+
+        https_pool = HTTPSConnectionPool('127.0.0.1', self.port,
+                                         cert_reqs='CERT_REQUIRED',
+                                         ca_certs=DEFAULT_CA)
+        self.addCleanup(https_pool.close)
+        r = https_pool.request('GET', '/')
+        self.assertEqual(r.status, 200)
+
 
 class TestHTTPS_IPv6Addr(IPV6HTTPSDummyServerTestCase):
     certs = IPV6_ADDR_CERTS
@@ -481,6 +569,7 @@ class TestHTTPS_IPv6Addr(IPV6HTTPSDummyServerTestCase):
         https_pool = HTTPSConnectionPool('[::1]', self.port,
                                          cert_reqs='CERT_REQUIRED',
                                          ca_certs=IPV6_ADDR_CA)
+        self.addCleanup(https_pool.close)
         r = https_pool.request('GET', '/')
         self.assertEqual(r.status, 200)
 
